#include <Stepper.h>
#include "pitches.h"

#define LDR_THRESHOLD 35
#define TONE_THRESH 250
#define CALIBRATION_MAX_DIST 5
#define BUZZER_PIN 4
#define TONE_DETECTION_FACTOR 0.9
#define CALIBRATION_LED_PIN 13

const int stepsPerRevolution = 2038;

Stepper myStepper = Stepper(stepsPerRevolution, 8, 10, 9, 11);

int ldr_pins[] = {A0, A2, A5};
int ldr_vals[3];
int tones[] = {NOTE_A4, NOTE_E5, NOTE_E6};
int tone_thresh[3];

void setup() {
  Serial.begin(9600);

  // Cauta pragul de detectare a notelor pentru fiecare senzor
  for (int i =0; i < 3; i++) {
    while(millis() < (i + 1) * 3000) {
      int sensorValue = analogRead(ldr_pins[i]); 
      if (sensorValue > tone_thresh[i]){  
        tone_thresh[i] = sensorValue; 
      } 
    }
    Serial.print("Threshold is ");
    Serial.println(tone_thresh[i]);
    tone_thresh[i] = tone_thresh[i] * TONE_DETECTION_FACTOR;
    Serial.print("Adjusted threshold is ");
    Serial.println(tone_thresh[i]);
  }

  // Muta oglinda in pozitia initiala in care raza laserului cade pe primul senzor
  myStepper.setSpeed(2);

  int ldrValue = analogRead(A0);
  int previousLdrValue = ldrValue;

  // Cauta senzorul in dreapta
  int calibrate_cnt = CALIBRATION_MAX_DIST;

  while (calibrate_cnt > 0 && abs(ldrValue - previousLdrValue) < LDR_THRESHOLD) {
    myStepper.step(stepsPerRevolution/50);
    previousLdrValue = ldrValue;
    ldrValue = analogRead(A0);
    Serial.println(ldrValue);
    delay(150);
    calibrate_cnt--;
  }

  // Daca nu a fost gasit senzorul in dreapta, cauta-l in stanga
  if (calibrate_cnt <= 0) {
    calibrate_cnt = 3 * CALIBRATION_MAX_DIST;

    while (calibrate_cnt > 0 && abs(ldrValue - previousLdrValue) < LDR_THRESHOLD) {
      myStepper.step(-stepsPerRevolution/50);
      previousLdrValue = ldrValue;
      ldrValue = analogRead(A0);
      Serial.println(ldrValue);
      delay(150);
      calibrate_cnt--;
    }
  }

  // Configureaza viteza motorului pentru blocul principal
  myStepper.setSpeed(6);
	delay(100);
}

void loop() {
  // Verifica daca raza a fost intrerupta pentru senzorul din stanga
  int newVal = analogRead(ldr_pins[0]);
  Serial.print("val 0 is ");
  Serial.println(ldr_vals[0]);

  if (ldr_vals[0] != 0 && newVal <= tone_thresh[0]) {
    tone(BUZZER_PIN, tones[0]);
  } else {
    noTone(BUZZER_PIN);
  }
  ldr_vals[0] = newVal;

  // Muta oglinda pe urmatorul senzor
	myStepper.step(stepsPerRevolution/64);
	delay(150);


  // Verifica daca raza a fost intrerupta pentru senzorul din mijloc
  newVal = analogRead(ldr_pins[1]);

  if (ldr_vals[1] != 0 && newVal <=  tone_thresh[1]) {
    tone(BUZZER_PIN, tones[1]);
  }else {
    noTone(BUZZER_PIN);
  }
  ldr_vals[1] = newVal;

  // Muta oglinda pe urmatorul senzor
	myStepper.step(stepsPerRevolution/64);
  delay(150);


  // Verifica daca raza a fost intrerupta pentru senzorul din dreapta  
  newVal = analogRead(ldr_pins[2]);

  if (ldr_vals[2] != 0 && newVal <=  tone_thresh[2]) {
    tone(BUZZER_PIN, tones[2]);
  }else {
    noTone(BUZZER_PIN);
  }
  ldr_vals[2] = newVal;

	myStepper.step(-stepsPerRevolution/64);
	delay(150);


  // Verifica daca raza a fost intrerupta pentru senzorul din mijloc
  newVal = analogRead(ldr_pins[1]);

  if (ldr_vals[1] != 0 && newVal <=  tone_thresh[1]) {
    tone(BUZZER_PIN, tones[1]);
  } else {
    noTone(BUZZER_PIN);
  }
  ldr_vals[1] = newVal;

  // Muta oglinda pe urmatorul senzor
	myStepper.step(-stepsPerRevolution/64);
  delay(150);
}

